/*
Copyright (C) 2009 -  Claudio Salvadori and Christian Nastasi
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *  \file hv7131gp_hal.h
 *
 *  \date Created on: 6-feb-2009
 *  \author Author: Claudio Salvadori and Christian Nastasi
 *  \brief
 *  Hardware abstraction level of CMOS Image Sensor HV7131GP using FLEX
 *  (with dsPIC33FJ256).
 */
#include "hv7131gp.h"

#ifndef HV7131GP_HAL_H_
#define HV7131GP_HAL_H_

#include "i2c.h"
#include "ee.h"
#include "cpu/pic30/inc/ee_irqstub.h"

/******************************************************************************/
/*                         Global Variables                                   */
/******************************************************************************/


/******************************************************************************/
/*                           Hardware Abstraction Layer                       */
/******************************************************************************/
/**
* @name Enable pin
* \brief Settings for D2 pin used like camera enable pin (see HV7131GP documentation)
*/
#define HV7131GP_PIN_ENABLE_INIT() 	TRISDbits.TRISD2=0	/* Init enable pin */
#define HV7131GP_PIN_ENABLE_HIGH() 	LATDbits.LATD2=1	/* Set pin to high value*/
#define HV7131GP_PIN_ENABLE_LOW() 	LATDbits.LATD2=0	/* Set pin to low value*/
/**  @} */

/**
* @name Reset pin
* \brief Settings for D1 pin used like camera reset pin (see HV7131GP documentation)
*/
#define HV7131GP_PIN_RESETB_INIT() 	TRISDbits.TRISD1=0	/* Init reset pin */
#define HV7131GP_PIN_RESETB_HIGH() 	LATDbits.LATD1=1	/* Set pin to high value*/
#define HV7131GP_PIN_RESETB_LOW() 	LATDbits.LATD1=0	/* Set pin to low value*/
/**  @} */

/**
* @name Frame debug pin
* \brief Settings for G1 pin used like debug: change state each frame captured
*/
#ifdef Debug
#define HV7131GP_PIN_DEBUG_INIT()  	TRISGbits.TRISG1 = 0 /* Init debug pin */
#endif
/**  @} */

/**
* @name Vertical Sync
*
* INT3. Functions to set, reset, start, stop the vertical sync interrupt
*
* @{ */
#define HV7131GP_PIN_VSYNC_INIT_POSITIVE()					\
do { 										\
	INTCON2bits.INT3EP = 0;	/*  Extern Interrupt on positive edge */	\
	IPC13bits.INT3IP  = 6;	/*  Set INT3 interrupt priority */		\
} while (0)									\

#define HV7131GP_PIN_VSYNC_INIT_NEGATIVE()					\
do { 										\
	INTCON2bits.INT3EP = 1;	/*  Extern Interrupt on negative edge */	\
	IPC13bits.INT3IP  = 6;	/*  Set INT3 interrupt priority */		\
} while (0)									\

#define HV7131GP_PIN_VSYNC_START()						\
do { 										\
	IEC3bits.INT3IE = 1;	/*  Enable INT3 interrupt */			\
	IFS3bits.INT3IF = 0;	/*  Reset INT3 interrupt flag */	 	\
} while (0)									\

#define HV7131GP_PIN_VSYNC_STOP()						\
do { 										\
	IEC3bits.INT3IE = 0;	/*  Disable INT3 interrupt */			\
	IFS3bits.INT3IF = 0;	/*  Reset INT3 interrupt flag */ 		\
} while (0)									\

#define HV7131GP_PIN_VSYNC_IF_RESET()						\
do { 										\
	IFS3bits.INT3IF = 0;	/*  Reset INT3 interrupt flag */	 	\
} while (0)									\

#define HV7131GP_PIN_VSYNC_IF_VALUE	IFS3bits.INT3IF /*  Return INT3 interrupt flag value*/
/**  @} */

/**
* @name Horizontal Sync
*
* CN15 (RD6). Functions to set, reset, start, stop the horizontal sync interrupt
*
* @{ */
#define HV7131GP_PIN_HSYNC_INIT()						\
do { 										\
	TRISDbits.TRISD6 = 1;	/* Set CN15 (RD6) as digital input */		\
	IPC4bits.CNIP = 6;	/* Set CN interrupt priority */			\
} while (0)									\

#define HV7131GP_PIN_HSYNC_START()						\
do { 										\
	CNEN1bits.CN15IE = 1;	/* Enable CN15(RD6) change interrupt */		\
	IEC1bits.CNIE = 1;	/* Enable CN interrupt */			\
	IFS1bits.CNIF = 0;	/* Reset CN interrupt flag */ 			\
} while (0)									\

#define HV7131GP_PIN_HSYNC_STOP()						\
do { 										\
	IEC1bits.CNIE = 0;	/* Disable CN interrupt */			\
	CNEN1bits.CN15IE = 0;	/* Disable CN15(RD6) change interrupt */	\
	IFS1bits.CNIF = 0;	/* Reset CN interrupt flag */	 		\
} while (0)									\

/**  @} */

/**
* @name Video (Pixels) Clock
*
* INT4. It's the pixel interrupt: everytime this interrupt occurs a pixel is
* captured.
*
* @{ */
#define HV7131GP_PIN_VCLK_INIT()						\
do { 										\
	INTCON2bits.INT4EP = 0;	/* Extern Interrupt on positive edge */		\
	IPC13bits.INT4IP  = 6;	/* Set INT4 interrupt priority */		\
} while (0)									\

#define HV7131GP_PIN_VCLK_START()						\
do { 										\
	IEC3bits.INT4IE = 1;	/*Enable INT4 interrupt */			\
	IFS3bits.INT4IF = 0;	/* Reset INT4 interrupt flag */	 		\
} while (0)									\

#define HV7131GP_PIN_VCLK_STOP()						\
do { 										\
	IEC3bits.INT4IE = 0;	/* Disable INT4 interrupt */			\
	IFS3bits.INT4IF = 0;	/* Reset INT4 interrupt flag */	 		\
} while (0)									\

/**  @} */


/**
* @name Master Clock
*
* OC1 with Timer3. The output of this pin is the master clock for HV7131GP camera.
* The master clock is the time reference of the camera. It's set a master clock
* with a frequency of 10MHz
*
* @{ */

#define HV7131GP_PIN_MCLK_INIT()	oc1_init()		/* Init master clock */
#define HV7131GP_PIN_MCLK_START		T3CONbits.TON = 1;	/* Start master clock */

/**
* @brief Stop OC1 Interface
*/
#define HV7131GP_PIN_MCLK_STOP()						\
do{ 										\
	T3CON = 0;		/* Stops Timer3 and reset control reg */	\
	TMR3 = 0;		/* Clear the timer register */			\
	LATDbits.LATD0 = 0;	/* Set line to low level */			\
} while (0)									\
/**  @} */

/**
* @name Pixel pins. In this set of 8 pins are received the 8 bits of each pixel.
* The pixel reception is regulated by the pixel clock
* @{ */
#define HV7131GP_PIN_Y_INIT()	TRISE = 0x00FF 	/* Init PORT_E */
#define HV7131GP_PIN_Y_VALUE	PORTE		/* Returns PORT_E */
/**  @} */

/**
* @name Timeout used by init camera function
* Timer5. Timer5 checks the timeout for init function based on the init timing graph
* (see HV7131GP ref man, pag. 9)
* \see hv7131gp_init
* @{ */
#define HV7131GP_TIMEOUT_INIT()		timer5_timeout_init() 	/* Init Timer5 */
#define HV7131GP_TIMEOUT_SET(t)		timer5_timeout_set((t)) /* Set timeout */


#define HV7131GP_TIMEOUT_RESET()/* Stop timer5 for timeout */			\
do{ 										\
	T5CONbits.TON = 0;	/* Stop Timer5 */				\
	IFS1bits.T5IF = 0;	/* Clear Timer5 interrupt status flag */	\
	IEC1bits.T5IE = 0;	/* Disable Timer5 interrupts */			\
} while (0)									\
/**  @} */
/**
* @name Software interrupt: end of frame acquisition
*
* INT0. This interrupt is generated when an entire frame is captured. This
* interrupt call inside his ISR the callback function passed on the capture
* function
* \see hv7131gp_capture
*
* @{ */
#define HV7131GP_PIN_EOF_INIT()						\
do { 									\
	INTCON2bits.INT0EP = 1;	/* Extern Interrupt on negative edge */	\
	IPC0bits.INT0IP  = 6;	/* Set INT0 interrupt priority */	\
	IEC0bits.INT0IE = 1;	/* Enable INT0 interrupt */		\
	IFS0bits.INT0IF = 0;	/* Reset INT0 interrupt flag */	 	\
} while (0)								\

#define HV7131GP_PIN_EOF_STOP()						\
do { 									\
	IEC0bits.INT0IE = 0;	/* Disable INT0 interrupt */		\
	IFS0bits.INT0IF = 0;	/* Reset INT0 interrupt flag */	 	\
} while (0)								\

/**  @} */

#endif /* HV7131GP_HAL_H_ */
